@@ -17,6 +17,13 @@
 # define CDECL extern "C"
 # define CDECL_BEGIN CDECL {
 # define CDECL_END   }
+template<typename T>
+static inline T replace_value(T &var,T new_value)
+{
+   T tmp=var;
+   var=new_value;
+   return tmp;
+}
 #else
 # define CDECL
 # define CDECL_BEGIN
@@ -112,6 +119,11 @@ CDECL void SOCKSinit(const char *);
 CDECL char *strtok_r(char *str, const char *sep, char **lasts);
 #endif
 
+#if !HAVE_DECL_MEMMEM
+CDECL void *memmem(const void *haystack, size_t haystack_len,
+		   const void *needle, size_t needle_len);
+#endif
+
 /* Tell the compiler when a conditional or integer expression is
    almost always true or almost always false.  */
 #ifndef HAVE_BUILTIN_EXPECT
@@ -142,4 +154,6 @@ CDECL char *strtok_r(char *str, const char *sep, char **lasts);
 # include "trio.h"
 #endif
 
+#define USE_SSL (defined(USE_GNUTLS) || defined(USE_OPENSSL))
+
 #endif /* POST_CONFIG_H */
