@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.cc,v 1.98 2002/07/31 14:40:02 lav Exp $ */
+/* $Id: CmdExec.cc,v 1.123 2006/08/02 13:44:26 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -37,6 +37,7 @@
 #include "QueueFeeder.h"
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
+#include "DummyProto.h"
 
 #define RL_PROMPT_START_IGNORE	'\001'
 #define RL_PROMPT_END_IGNORE	'\002'
@@ -52,6 +53,7 @@ static ResDecl
    res_default_ls	   ("cmd:ls-default",	"",0,0),
    res_csh_history	   ("cmd:csh-history",	"off",ResMgr::BoolValidate,ResMgr::NoClosure),
    res_verify_path	   ("cmd:verify-path",	"yes",ResMgr::BoolValidate,0),
+   res_verify_path_cached  ("cmd:verify-path-cached","no",ResMgr::BoolValidate,0),
    res_verify_host	   ("cmd:verify-host",	"yes",ResMgr::BoolValidate,0),
    res_at_exit		   ("cmd:at-exit",	"",   0,0),
    res_fail_exit	   ("cmd:fail-exit",	"no", ResMgr::BoolValidate,ResMgr::NoClosure),
@@ -59,10 +61,12 @@ static ResDecl
    res_interactive	   ("cmd:interactive",	"no", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_move_background	   ("cmd:move-background","yes", ResMgr::BoolValidate,ResMgr::NoClosure),
    res_set_term_status     ("cmd:set-term-status","no", ResMgr::BoolValidate,0),
-   res_term_status         ("cmd:term-status",  "", 0, 0);
+   res_term_status         ("cmd:term-status",  "", 0, 0),
+   res_trace		   ("cmd:trace",  "no",	ResMgr::BoolValidate,ResMgr::NoClosure);
 
-CmdExec	 *CmdExec::cwd_owner=0;
-CmdExec	 *CmdExec::chain=0;
+CmdExec	 *CmdExec::cwd_owner;
+CmdExec	 *CmdExec::chain;
+CmdExec  *CmdExec::top;
 
 void  CmdExec::SaveCWD()
 {
@@ -93,7 +97,7 @@ int  CmdExec::RestoreCWD()
 void CmdExec::FeedCmd(const char *c)
 {
    partial_cmd=false;
-   time(&start_time);
+   start_time=now;
    if(cmd_buf==0)
    {
       cmd_buf=next_cmd=xstrdup(c);
@@ -107,7 +111,7 @@ void CmdExec::FeedCmd(const char *c)
 
 void CmdExec::PrependCmd(const char *c)
 {
-   time(&start_time);
+   start_time=now;
 
    int len=strlen(c);
    int nl=(len>0 && c[len-1]!='\n');
@@ -187,6 +191,13 @@ void  CmdExec::exec_parsed_command()
 
    bool did_default=false;
 
+   if(ResMgr::QueryBool("cmd:trace",0))
+   {
+      char *c=args->CombineQuoted();
+      printf("+ %s\n",c);
+      xfree(c);
+   }
+
 restart:
 
    const struct cmd_rec *c;
@@ -205,6 +216,7 @@ restart:
       }
 
       args->setarg(0,c->name); // in case it was abbreviated
+      args->rewind();
 
       if(cmd==0)
 	 cmd=args->Combine();
@@ -271,22 +283,20 @@ void CmdExec::SuspendJob(Job *j)
 
 void CmdExec::ExecParsed(ArgV *a,FDStream *o,bool b)
 {
-   bool old_running=running;
-   running=true; // disable reentering
+   Enter();
 
    if(args)
       delete args;
    args=a;
    xfree(cmd);
    cmd=args->Combine();
-   if(output)
-      delete output;
+   delete output;
    output=o;
    background=b;
    condition=COND_ANY;
    exec_parsed_command();
 
-   running=old_running;
+   Leave();
 }
 
 bool CmdExec::Idle()
@@ -308,6 +318,10 @@ void CmdExec::RemoveFeeder()
    if(!feeder)
       return;
 
+   // save old cwd if necessary
+   if(interactive && feeder->prev==0)
+      cwd_history.Set(session);
+
    CmdFeeder *tmp=feeder->prev;
    next_cmd=cmd_buf=feeder->saved_buf;
    partial_cmd=false;
@@ -316,9 +330,6 @@ void CmdExec::RemoveFeeder()
    delete feeder;
    feeder=tmp;
 
-   // save old cwd if necessary
-   if(interactive && feeder==0)
-      cwd_history.Set(session,session->GetCwd());
 }
 
 void CmdExec::ReuseSavedSession()
@@ -368,9 +379,9 @@ int CmdExec::Do()
 	       const char *cwd=session->GetCwd();
 	       eprintf(_("cd ok, cwd=%s\n"),cwd?cwd:"~");
 	       cwd_history.Set(session,old_cwd);
-	       if(slot)
-		  ConnectionSlot::SetCwd(slot,cwd);
 	    }
+	    if(slot)
+	       ConnectionSlot::SetCwd(slot,session->GetCwd());
 	    session->Close();
 	    exit_code=0;
 	    builtin=BUILTIN_NONE;
@@ -385,8 +396,7 @@ int CmdExec::Do()
 	       // cd to another url.
 	       const char *loc_c=session->GetNewLocation();
 	       int max_redirections=ResMgr::Query("xfer:max-redirections",0);
-	       if(loc_c && loc_c[0] && max_redirections>0
-	       && loc_c[strlen(loc_c)-1]=='/')
+	       if(loc_c && max_redirections>0)
 	       {
 		  eprintf(_("%s: received redirection to `%s'\n"),"cd",loc_c);
 		  if(++redirections>max_redirections)
@@ -396,17 +406,18 @@ int CmdExec::Do()
 		  }
 
 		  char *loc=alloca_strdup(loc_c);
-		  session->Close(); // loc_c is no longer valid.
-
 		  ParsedURL u(loc,true);
-
 		  if(!u.proto)
 		  {
-		     url::decode_string(loc);
-		     session->Chdir(loc);
+		     bool is_file=(last_char(loc)!='/');
+		     FileAccess::Path new_cwd(session->GetNewCwd());
+		     new_cwd.Change(0,is_file,loc);
+		     session->PathVerify(new_cwd);
 		     Roll(session);
 		     return MOVED;
 		  }
+		  session->Close();
+		  exit_code=0;
 		  builtin=BUILTIN_NONE;
 		  char *cmd=string_alloca(6+3+2*strlen(loc));
 		  strcpy(cmd,"open \"");
@@ -474,11 +485,11 @@ int CmdExec::Do()
 	 }
 	 if(glob->Done() || glob->Error())
 	 {
-	    delete glob;
-	    glob=0;
 	    const char *pat=args->getnext();
 	    if(!pat)
 	    {
+	       delete glob;
+	       glob=0;
 	       // it was last argument
 	       delete args;
 	       args=args_glob;
@@ -491,7 +502,7 @@ int CmdExec::Do()
 	       exec_parsed_command();
 	       return MOVED;
 	    }
-	    glob=new GlobURL(session,pat);
+	    glob->NewGlob(pat);
 	    m=MOVED;
 	 }
 	 break;
@@ -515,12 +526,7 @@ int CmdExec::Do()
 	       if(status_line)
 		  status_line->Clear();
 	       if(builtin==BUILTIN_CD)
-	       {
-		  // accept the path
-		  const char *f=session->GetFile();
-		  f=alloca_strdup(f);
-		  session->Chdir(f,false);
-	       }
+		  session->ChdirAccept();
 	       session->Close();
 	       exit_code=0;
 	       builtin=BUILTIN_NONE;
@@ -538,7 +544,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line)
+      if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       return m;
    }
@@ -582,7 +588,7 @@ int CmdExec::Do()
 	    return MOVED;
 	 }
       }
-      if(status_line)
+      if(status_line && status_line->CanShowNow())
 	 ShowRunStatus(status_line);   // this is only for top level CmdExec.
       return m;
    }
@@ -616,13 +622,15 @@ try_get_cmd:
 
 	 char *prompt=MakePrompt();
 	 feeder_called=true;
+	 if(fg)
+	    feeder->Fg();
 	 const char *cmd=feeder->NextCmd(this,prompt);
 	 feeder_called=false;
 	 if(cmd==0)
 	 {
 	    if(next_cmd && *next_cmd && partial_cmd)
 	    {
-	       if(next_cmd[strlen(next_cmd)-1]!='\n')
+	       if(last_char(next_cmd)!='\n')
 	       {
 		  // missing EOL on last line, add it
 		  FeedCmd("\n");
@@ -673,6 +681,8 @@ try_get_cmd:
       partial_cmd=true;
       goto try_get_cmd;
    case(PARSE_OK):
+      if(feeder)
+	 feeder->Bg();
       break;
    }
    if(args==0 || args->count()==0)
@@ -692,7 +702,7 @@ void CmdExec::ShowRunStatus(StatusLine *s)
    {
    case(BUILTIN_CD):
       if(session->IsOpen())
-	 s->Show("cd `%s' [%s]",args->getarg(1),session->CurrentStatus());
+	 s->Show("cd `%s' [%s]",squeeze_file_name(args->getarg(1),s->GetWidthDelayed()-40),session->CurrentStatus());
       break;
    case(BUILTIN_OPEN):
       if(session->IsOpen())
@@ -724,8 +734,9 @@ void CmdExec::PrintStatus(int v,const char *prefix)
    }
    if(queue_feeder)
    {
-      if(suspended)
+      if(IsSuspended())
 	 printf("%s%s\n",prefix,_("Queue is stopped."));
+      BuryDoneJobs();
       for(int i=0; i<waiting_num; i++)
       {
 	 if(i==0)
@@ -766,7 +777,7 @@ void CmdExec::PrintStatus(int v,const char *prefix)
    }
 }
 
-CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f)
+CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f?f:new DummyProto)
 {
    // add this to chain
    next=chain;
@@ -794,14 +805,16 @@ CmdExec::CmdExec(FileAccess *f,LocalDirectory *c) : SessionJob(f)
    last_bg=-1;
 
    cwd=c;
+   if(!cwd)
+      SaveCWD();
 
    remote_completion=false;
    long_running=0;
    csh_history=false;
    verify_host=verify_path=true;
+   verify_path_cached=false;
 
    start_time=0;
-   old_cwd=0;
    old_lcwd=0;
    slot=0;
 
@@ -839,7 +852,6 @@ CmdExec::~CmdExec()
    delete cwd;
    if(cwd_owner==this)
       cwd_owner=0;
-   xfree(old_cwd);
    xfree(old_lcwd);
    delete glob;
    delete args_glob;
@@ -922,7 +934,7 @@ char *CmdExec::MakePrompt()
 void CmdExec::beep_if_long()
 {
    if(start_time!=0 && long_running!=0
-   && time(0)-start_time>long_running
+   && now.UnixTime()>start_time+long_running
    && interactive && Idle() && isatty(1))
       write(1,"\007",1);
 }
@@ -937,6 +949,7 @@ void CmdExec::Reconfig(const char *name)
    remote_completion = res_remote_completion.QueryBool(c);
    csh_history = res_csh_history.QueryBool(0);
    verify_path=res_verify_path.QueryBool(c);
+   verify_path_cached=res_verify_path_cached.QueryBool(c);
    verify_host=res_verify_host.QueryBool(c);
    verbose=res_verbose.QueryBool(0);
    // only allow explicit setting of cmd:interactive to change interactiveness.
@@ -1100,11 +1113,12 @@ char *ArgV::CombineQuoted(int start) const
 {
    int	 i;
    char  *res;
-   char	 *store,*arg;
+   char	 *store;
+   const char *arg;
    int	 len=0;
 
-   for(i=start; i<c; i++)
-      len+=strlen(v[i])*2+3;
+   for(i=start; i<Count(); i++)
+      len+=strlen(String(i))*2+3;
 
    if(len==0)
       return(xstrdup(""));
@@ -1112,9 +1126,9 @@ char *ArgV::CombineQuoted(int start) const
    res=(char*)xmalloc(len);
 
    store=res;
-   for(i=start; i<c; i++)
+   for(i=start; i<Count(); i++)
    {
-      arg=v[i];
+      arg=String(i);
       if(CmdExec::needs_quotation(arg))
       {
 	 *store++='"';
@@ -1219,7 +1233,7 @@ void CmdExec::RegisterCommand(const char *name,cmd_creator_t creator,const char
 {
    if(dyn_cmd_table==0)
    {
-      dyn_cmd_table_count=1;
+      dyn_cmd_table_count=2;
       for(const cmd_rec *c=static_cmd_table; c->name; c++)
 	 dyn_cmd_table_count++;
       dyn_cmd_table=(cmd_rec*)xmemdup(static_cmd_table,dyn_cmd_table_count*sizeof(cmd_rec));
@@ -1297,19 +1311,21 @@ void CmdExec::FeedArgV(const ArgV *args,int start)
    xfree(cmd);
 }
 
-bool CmdExec::SameQueueParameters(CmdExec *scan)
+bool CmdExec::SameQueueParameters(CmdExec *scan,const char *this_url)
 {
-   return !strcmp(this->session->GetConnectURL(FA::NO_PATH),
-	          scan->session->GetConnectURL(FA::NO_PATH))
+   return !strcmp(this_url,scan->session->GetConnectURL(FA::NO_PATH))
       && !xstrcmp(this->slot,scan->slot);
 }
 
 /* return the CmdExec containing a queue feeder; create if necessary */
 CmdExec  *CmdExec::GetQueue(bool create)
 {
+   const char *this_url=session->GetConnectURL(FA::NO_PATH);
+   // future GetConnectURL overwrite the static buffer, save it.
+   this_url=alloca_strdup(this_url);
    for(CmdExec *scan=chain; scan; scan=scan->next)
    {
-      if(scan->queue_feeder && SameQueueParameters(scan))
+      if(scan->queue_feeder && SameQueueParameters(scan,this_url))
 	 return scan;
    }
    if(!create)
