@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CmdExec.h,v 1.55 2006/04/05 12:28:15 lav Exp $ */
+/* $Id: CmdExec.h,v 1.56 2006/08/02 13:44:26 lav Exp $ */
 
 #ifndef CMDEXEC_H
 #define CMDEXEC_H
@@ -138,7 +138,7 @@ private:
 
    QueueFeeder *queue_feeder;
    CmdExec *GetQueue(bool create = true);
-   bool SameQueueParameters(CmdExec *);
+   bool SameQueueParameters(CmdExec *,const char *);
 
    FileAccess *saved_session;
    void ReuseSavedSession();
