@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CopyJob.cc,v 1.30 2002/09/02 10:47:36 lav Exp $ */
+/* $Id: CopyJob.cc,v 1.33 2006/02/06 11:03:58 lav Exp $ */
 
 #include <config.h>
 #include "CopyJob.h"
@@ -167,6 +167,32 @@ CopyJob::~CopyJob()
    xfree(op);
 }
 
+const char *CopyJob::FormatBytesTimeRate(off_t bytes,double time_spent)
+{
+   static char buf[256];
+
+   if(bytes<=0)
+      return "";
+
+   if(time_spent>=1)
+   {
+      sprintf(buf,plural("%lld $#ll#byte|bytes$ transferred"
+		     " in %ld $#l#second|seconds$",
+		     (long long)bytes,long(time_spent+.5)),
+		     (long long)bytes,long(time_spent+.5));
+      double rate=bytes/time_spent;
+      if(rate>=1)
+	 sprintf(buf+strlen(buf)," (%s)",Speedometer::GetStr(rate));
+   }
+   else
+   {
+      sprintf(buf,plural("%lld $#ll#byte|bytes$ transferred",
+		     (long long)bytes),(long long)bytes);
+   }
+   return buf;
+}
+
+
 // CopyJobEnv
 CopyJobEnv::CopyJobEnv(FileAccess *s,ArgV *a,bool cont1)
    : SessionJob(s)
@@ -248,26 +274,7 @@ void CopyJobEnv::SayFinalWithPrefix(const char *p)
    if(count==errors)
       return;
    if(bytes)
-   {
-      printf("%s",p);
-      if(time_spent>=1)
-      {
-	 printf(plural("%lld $#ll#byte|bytes$ transferred"
-			" in %ld $#l#second|seconds$",
-			(long long)bytes,long(time_spent+.5)),
-			(long long)bytes,long(time_spent+.5));
-	 double rate=bytes/time_spent;
-	 if(rate>=1)
-	    printf(" (%s)\n",Speedometer::GetStr(rate));
-	 else
-	    printf("\n");
-      }
-      else
-      {
-	 printf(plural("%lld $#ll#byte|bytes$ transferred\n",
-			(long long)bytes),(long long)bytes);
-      }
-   }
+      printf("%s%s\n",p,CopyJob::FormatBytesTimeRate(bytes,time_spent));
    if(errors>0)
    {
       printf("%s",p);
