@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.h,v 1.63 2005/11/29 14:37:34 lav Exp $ */
+/* $Id: FileCopy.h,v 1.69 2006/07/04 05:15:36 lav Exp $ */
 
 /*
   classes defined here:
@@ -39,8 +39,6 @@
 #include "Speedometer.h"
 #include "Timer.h"
 
-#define FILE_END     (-1L)
-
 class FileCopyPeer : public IOBuffer
 {
 protected:
@@ -98,7 +96,11 @@ public:
    void DontCopyDate() { do_set_date=false; }
    bool NeedDate() { return do_set_date; }
 
-   void SetRange(off_t s,off_t lim) { range_start=s; range_limit=lim; }
+   void SetRange(off_t s,off_t lim) {
+      range_start=s; range_limit=lim;
+      if(range_start>GetPos()+0x4000);
+	 Seek(range_start);
+   }
 
    FileCopyPeer(dir_t m);
 
@@ -144,6 +146,7 @@ public:
 	 if(f && !suggested_filename)
 	    suggested_filename=xstrdup(f);
       }
+   void AutoRename(bool yes=true) { auto_rename=yes; }
 };
 
 class FileCopy : public SMTask
@@ -235,6 +238,7 @@ public:
    void DontFailIfBroken() { fail_if_broken=false; }
    void FailIfCannotSeek() { fail_if_cannot_seek=true; }
    void SetRange(off_t s,off_t lim) { get->SetRange(s,lim); put->SetRange(s,lim); }
+   void SetRangeLimit(off_t lim) { get->range_limit=lim; }
    void RemoveSourceLater() { remove_source_later=true; }
    void RemoveTargetFirst() { remove_target_first=true; put->Resume(); put->RemoveFile(); }
    void LineBuffered(int size=0x1000);
@@ -321,9 +325,10 @@ public:
    void DontReuseSession() { reuse_later=false; }
 
    const char *GetStatus();
-   const char *GetProto() { return session->GetProto(); }
+   const char *GetProto() const { return session->GetProto(); }
 
    bool NeedSizeDateBeforehand() { return session->NeedSizeDateBeforehand(); }
+   void WantSize();
    void RemoveFile();
 
    static FileCopyPeerFA *New(FA *s,const char *url,int m,bool reuse=false);
@@ -374,6 +379,7 @@ public:
    void DontDeleteStream() { delete_stream=false; }
    void DontCreateFgData() { create_fg_data=false; }
    void NeedSeek() { need_seek=true; }
+   void WantSize();
    void RemoveFile();
    void SetBase(off_t b) { seek_base=b; }
 
