@@ -24,6 +24,7 @@
 #include "FindJobDu.h"
 #include "CmdExec.h"
 #include "misc.h"
+#include "buffer_std.h"
 
 CDECL_BEGIN
 #include "human.h"
@@ -35,7 +36,11 @@ FinderJob_Du::FinderJob_Du(FileAccess *s,ArgV *a,FDStream *o):
    args=a;
    op=args->a0();
 
-   buf=new IOBufferFDStream(o,IOBuffer::PUT);
+   if(o)
+      buf=new IOBufferFDStream(o,IOBuffer::PUT);
+   else
+      buf=new IOBuffer_STDOUT(this);
+   show_sl = !o || !o->usesfd(1);
 
    Need(FileInfo::SIZE);
 
@@ -43,6 +48,7 @@ FinderJob_Du::FinderJob_Du(FileAccess *s,ArgV *a,FDStream *o):
    max_print_depth = -1;
    print_totals = false;
    output_block_size = 1024;
+   human_opts = 0;
    all_files = false;
    separate_dirs = false;
    file_count = false;
@@ -117,7 +123,7 @@ FinderJob::prf_res FinderJob_Du::ProcessFile(const char *d,const FileInfo *fi)
       return PRF_OK; /* can't count this one */
 
    /* add this file to the current dir */
-   int add = fi->size;
+   long long add = fi->size;
    if (file_count)
       add = 1;
    if(stack_ptr != -1)
@@ -172,8 +178,7 @@ void FinderJob_Du::print_size (long long n_blocks, const char *string)
    /* We get blocks in bytes, since we don't know the remote system's
     * block size. */
    buf->Format("%s\t%s\n",
-	 human_readable_inexact (n_blocks, buffer, 1,
-	    output_block_size, human_ceiling),
+	 human_readable (n_blocks, buffer, human_opts, 1, output_block_size),
 	 string);
 }
 
