@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2006 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,15 +70,11 @@ int Fish::Do()
    int fd;
 
    // check if idle time exceeded
-   if(mode==CLOSED && send_buf && idle>0)
+   if(mode==CLOSED && send_buf && idle_timer.Stopped())
    {
-      if(now >= idle_start+idle)
-      {
-	 DebugPrint("---- ",_("Closing idle connection"),1);
-	 Disconnect();
-	 return m;
-      }
-      TimeoutS(idle_start+idle-time_t(now));
+      DebugPrint("---- ",_("Closing idle connection"),1);
+      Disconnect();
+      return m;
    }
 
    if(Error())
@@ -98,14 +94,15 @@ int Fish::Do()
       return m;
 
    if(send_buf)
-      BumpEventTime(send_buf->EventTime());
+      timeout_timer.Reset(send_buf->EventTime());
    if(recv_buf)
-      BumpEventTime(recv_buf->EventTime());
+      timeout_timer.Reset(recv_buf->EventTime());
 
    if((state==FILE_RECV || state==FILE_SEND)
    && rate_limit==0)
       rate_limit=new RateLimit(hostname);
 
+   const char *charset;
    switch(state)
    {
    case DISCONNECTED:
@@ -162,7 +159,7 @@ int Fish::Do()
       }
       ssh=new PtyShell(cmd);
       state=CONNECTING;
-      event_time=now;
+      timeout_timer.Reset();
       m=MOVED;
    }
    case CONNECTING:
@@ -188,6 +185,14 @@ int Fish::Do()
    case CONNECTING_1:
       if(!received_greeting)
 	 return m;
+
+      charset=ResMgr::Query("fish:charset",hostname);
+      if(charset && *charset)
+      {
+	 send_buf->SetTranslation(charset,false);
+	 recv_buf->SetTranslation(charset,true);
+      }
+
       Send("#FISH\n"
 	   "exec 2>&1;echo;start_fish_server;"
 	   "TZ=GMT;export TZ;LC_ALL=C;export LC_ALL;"
@@ -280,9 +285,9 @@ usual_return:
    if(m==MOVED)
       return MOVED;
    if(send_buf)
-      BumpEventTime(send_buf->EventTime());
+      timeout_timer.Reset(send_buf->EventTime());
    if(recv_buf)
-      BumpEventTime(recv_buf->EventTime());
+      timeout_timer.Reset(recv_buf->EventTime());
    if(CheckTimeout())
       return MOVED;
 // notimeout_return:
@@ -302,7 +307,7 @@ void Fish::MoveConnectionHere(Fish *o)
    RQ_alloc=o->RQ_alloc; o->RQ_alloc=0;
    RQ_head=o->RQ_head; o->RQ_head=0;
    RQ_tail=o->RQ_tail; o->RQ_tail=0;
-   BumpEventTime(o->event_time);
+   timeout_timer.Reset(o->timeout_timer);
    set_real_cwd(o->real_cwd);
    o->set_real_cwd(0);
    state=CONNECTED;
@@ -555,10 +560,23 @@ void Fish::SendMethod()
 	 SetError(NO_FILE,"Have to know file size before upload");
 	 break;
       }
-      // dd pays attension to read boundaries and reads wrong number
-      // of bytes when ibs>1. Have to use the inefficient ibs=1.
       if(entity_size>0)
       {
+	 Send("#STOR %lld %s\n"
+	      "rest=%lld;file=%s;:>$file;echo '### 001';"
+	      "if echo 1|head -c 1 -q ->/dev/null 2>&1;then "
+		  "head -c $rest -q -|(cat>$file;cat>/dev/null);"
+	      "else while [ $rest -gt 0 ];do "
+		  "bs=4096;cnt=`expr $rest / $bs`;"
+		  "[ $cnt -eq 0 ] && { cnt=1;bs=$rest; }; "
+		  "n=`dd ibs=$bs count=$cnt 2>/dev/null|tee -a $file|wc -c`;"
+		  "[ \"$n\" -le 0 ] && exit;"
+		  "rest=`expr $rest - $n`; "
+	      "done;fi;echo '### 200'\n",
+	    (long long)entity_size,e,(long long)entity_size,e);
+#if 0
+	 // dd pays attension to read boundaries and reads wrong number
+	 // of bytes when ibs>1. Have to use the inefficient ibs=1.
 	 Send("#STOR %lld %s\n"
 	      ">%s;echo '### 001';"
 	      "dd ibs=1 count=%lld 2>/dev/null"
@@ -567,6 +585,7 @@ void Fish::SendMethod()
 	      e,
 	      (long long)entity_size,
 	      e);
+#endif
       }
       else
       {
@@ -674,7 +693,7 @@ int Fish::HandleReplies()
 	       SetError(LOGIN_FAILED,_("Password required"));
 	       return MOVED;
 	    }
-	    if(password_sent>0)
+	    if(password_sent>1)
 	    {
 	       SetError(LOGIN_FAILED,_("Login incorrect"));
 	       return MOVED;
@@ -755,7 +774,7 @@ int Fish::HandleReplies()
       PropagateHomeAuto();
       if(!home)
 	 set_home(home_auto);
-      LsCache::SetDirectory(this, home, true);
+      cache->SetDirectory(this, home, true);
       break;
    case EXPECT_CWD:
       p=PopDirectory();
@@ -767,7 +786,7 @@ int Fish::HandleReplies()
 	    cwd.Set(p);
 	    eof=true;
 	 }
-	 LsCache::SetDirectory(this,p,true);
+	 cache->SetDirectory(this,p,true);
       }
       else
 	 SetError(NO_FILE,message);
@@ -1131,7 +1150,7 @@ const char *Fish::CurrentStatus()
    return "";
 }
 
-bool Fish::SameSiteAs(FileAccess *fa)
+bool Fish::SameSiteAs(const FileAccess *fa) const
 {
    if(!SameProtoAs(fa))
       return false;
@@ -1140,7 +1159,7 @@ bool Fish::SameSiteAs(FileAccess *fa)
    && !xstrcmp(user,o->user) && !xstrcmp(pass,o->pass));
 }
 
-bool Fish::SameLocationAs(FileAccess *fa)
+bool Fish::SameLocationAs(const FileAccess *fa)	const
 {
    if(!SameSiteAs(fa))
       return false;
@@ -1170,6 +1189,17 @@ void Fish::CleanupThis()
 void Fish::Reconfig(const char *name)
 {
    super::Reconfig(name);
+   if(!xstrcmp(name,"fish:charset") && recv_buf && send_buf)
+   {
+      if(!IsSuspended())
+	 cache->TreeChanged(this,"/");
+      const char *charset=ResMgr::Query("fish:charset",hostname);
+      if(charset && *charset)
+      {
+	 send_buf->SetTranslation(charset,false);
+	 recv_buf->SetTranslation(charset,true);
+      }
+   }
 }
 
 void Fish::ClassInit()
@@ -1213,7 +1243,7 @@ int FishDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,&err,
+      if(use_cache && FileAccess::cache->Find(session,pattern,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
 	 if(err)
@@ -1230,8 +1260,8 @@ int FishDirList::Do()
 	 session->Open(pattern,FA::LONG_LIST);
 	 ((Fish*)session)->DontEncodeFile();
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -1241,7 +1271,7 @@ int FishDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-      LsCache::Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
+      FileAccess::cache->Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
       return MOVED;
    }
 
@@ -1323,7 +1353,7 @@ static FileSet *ls_to_FileSet(const char *b,int len)
    return set;
 }
 
-FileSet *Fish::ParseLongList(const char *b,int len,int *err)
+FileSet *Fish::ParseLongList(const char *b,int len,int *err) const
 {
    if(err)
       *err=0;
