@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpDirList.cc,v 1.23 2005/04/04 09:59:40 lav Exp $ */
+/* $Id: FtpDirList.cc,v 1.27 2006/06/13 14:35:36 lav Exp $ */
 
 #include <config.h>
 
@@ -56,7 +56,7 @@ int FtpDirList::Do()
       const char *cache_buffer=0;
       int cache_buffer_size=0;
       int err;
-      if(use_cache && LsCache::Find(session,pattern,FA::LONG_LIST,&err,
+      if(use_cache && FileAccess::cache->Find(session,pattern,FA::LONG_LIST,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
 	 if(err)
@@ -72,8 +72,8 @@ int FtpDirList::Do()
       {
 	 session->Open(pattern,FA::LONG_LIST);
 	 ubuf=new IOBufferFileAccess(session);
-	 if(LsCache::IsEnabled())
-	    ubuf->Save(LsCache::SizeLimit());
+	 if(FileAccess::cache->IsEnabled(session->GetHostName()))
+	    ubuf->Save(FileAccess::cache->SizeLimit());
       }
    }
 
@@ -83,7 +83,7 @@ int FtpDirList::Do()
    if(b==0) // eof
    {
       buf->PutEOF();
-      LsCache::Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
+      FileAccess::cache->Add(session,pattern,FA::LONG_LIST,FA::OK,ubuf);
       return MOVED;
    }
 
@@ -145,17 +145,15 @@ const char *FtpDirList::Status()
    return "";
 }
 
-void FtpDirList::Suspend()
+void FtpDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void FtpDirList::Resume()
+void FtpDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 void FtpDirList::FormatGeneric(FileInfo *fi)
