@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpListInfo.cc,v 1.60 2005/12/31 07:01:36 lav Exp $ */
+/* $Id: FtpListInfo.cc,v 1.62 2006/06/13 14:35:36 lav Exp $ */
 
 #include <config.h>
 #include "FtpListInfo.h"
@@ -61,7 +61,7 @@ FileSet *FtpListInfo::Parse(const char *buf,int len)
    }
 }
 
-FileSet *Ftp::ParseLongList(const char *buf,int len,int *err_ret)
+FileSet *Ftp::ParseLongList(const char *buf,int len,int *err_ret) const
 {
    if(err_ret)
       *err_ret=0;
