@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: GetJob.h,v 1.5 2000/09/07 15:20:32 lav Exp $ */
+/* $Id: GetJob.h,v 1.12 2006/06/30 14:10:27 lav Exp $ */
 
 #ifndef GETJOB_H
 #define GETJOB_H
@@ -27,10 +27,17 @@
 
 class GetJob : public CopyJobEnv
 {
+   FileCopyPeer *NoProtoSrcLocal(const char *src);
+   FileCopyPeer *NoProtoDstLocal(const char *dst);
+   FileCopyPeer *NoProtoPeer(FileAccess *session,const char *file,FA::open_mode mode);
+
 protected:
+   FileCopyPeer *CreateCopyPeer(FileAccess *session,const char *path,FA::open_mode mode);
    void	 NextFile();
 
    bool delete_files;
+   bool remove_target_first;
+   bool truncate_target_first;
    char *backup_file;
    mode_t file_mode;
    FileStream *local;
@@ -38,9 +45,6 @@ protected:
 
    void RemoveBackupFile();
 
-   FileCopyPeer *NoProtoSrc(const char *src);
-   FileCopyPeer *NoProtoDst(const char *dst);
-
 public:
    GetJob(FileAccess *s,ArgV *a,bool c=false);
    ~GetJob();
@@ -48,7 +52,9 @@ public:
    int	 Do();
 
    void DeleteFiles() { delete_files=true; }
+   void RemoveSourceLater() { delete_files=true; }
    void Reverse() { reverse=true; } // put
+   void RemoveTargetFirst() { remove_target_first=true; }
 };
 
 #endif /* GETJOB_H */
