@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.41 2005/04/05 09:49:41 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.45 2006/02/06 11:00:40 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -85,7 +85,7 @@ class MirrorJob : public Job
       Statistics();
       void Reset();
       void Add(const Statistics &);
-      bool HaveSomethingDone();
+      bool HaveSomethingDone(int);
    };
    Statistics stats;
 
@@ -93,6 +93,7 @@ class MirrorJob : public Job
    int	 &transfer_count;
 
    int	 flags;
+   int	 max_error_count;
 
    PatternSet *exclude;
 
@@ -104,6 +105,8 @@ class MirrorJob : public Job
    MirrorJob *parent_mirror;
 
    time_t newer_than;
+   time_t older_than;
+   Range *size_range;
 
    char *script_name;
    FILE *script;
@@ -167,16 +170,15 @@ public:
    void	 SayFinal() { PrintStatus(0,""); }
    int	 ExitCode() { return stats.error_count; }
 
-   void SetExclude(PatternSet *x)
-      {
-	 exclude=x;
-      }
+   void	 SetExclude(PatternSet *x) { exclude=x; }
+   void	 SetSizeRange(Range *r) { size_range=r; }
 
    void	 SetVerbose(int v) { verbose_report=v; }
 
    void	 CreateRemoteDir() { create_remote_dir=true; }
 
    void	 SetNewerThan(const char *file);
+   void	 SetOlderThan(const char *file);
 
    void  UseCache(bool u) { use_cache=u; }
    void	 RemoveSourceFiles() { remove_source_files=true; }
@@ -192,6 +194,7 @@ public:
       {
 	 script_only=yes;
       }
+   void SetMaxErrorCount(int ec) { max_error_count=ec; }
 };
 
 #endif//MIRRORJOB_H
