@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: PtyShell.cc,v 1.10 2004/02/17 10:40:56 lav Exp $ */
+/* $Id: PtyShell.cc,v 1.11 2005/04/29 10:29:59 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -150,7 +150,7 @@ int PtyShell::getfd()
       putenv((char*)"LANGUAGE=C");
       if(a)
 	 execvp(a->a0(),a->GetVNonConst());
-      execl("/bin/sh","sh","-c",name,NULL);
+      execl("/bin/sh","sh","-c",name,(char*)NULL);
       fprintf(stderr,_("execl(/bin/sh) failed: %s\n"),strerror(errno));
       fflush(stderr);
       _exit(1);
