@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.63 2005/01/21 10:33:16 lav Exp $ */
+/* $Id: Resolver.cc,v 1.70 2006/06/13 14:35:40 lav Exp $ */
 
 #include <config.h>
 
@@ -108,9 +108,8 @@ Resolver::Resolver(const char *h,const char *p,const char *defp,
    w=0;
    err_msg=0;
    done=false;
-   timeout=0;
+   timeout_timer.SetResource("dns:fatal-timeout",hostname);
    Reconfig();
-   time(&start_time);
    addr=0;
    addr_num=0;
    buf=0;
@@ -243,15 +242,11 @@ int   Resolver::Do()
 
    if(!buf->Eof())   // wait for all data to arrive (not too much)
    {
-      if(timeout>0)
+      if(timeout_timer.Stopped())
       {
-	 if(now >= start_time+timeout)
-	 {
-	    err_msg=xstrdup(_("host name resolve timeout"));
-	    done=true;
-	    return MOVED;
-	 }
-	 TimeoutS(timeout-(time_t(now)-start_time));
+	 err_msg=xstrdup(_("host name resolve timeout"));
+	 done=true;
+	 return MOVED;
       }
       return m;
    }
@@ -385,7 +380,7 @@ void Resolver::ParseOrder(const char *s,int *o)
    if(o) o[idx]=-1;
 }
 
-#if defined(HAVE_DN_EXPAND) && !defined(HAVE_DECL_DN_EXPAND)
+#if defined(HAVE_DN_EXPAND) && !HAVE_DECL_DN_EXPAND
 CDECL int dn_expand(const unsigned char *msg,const unsigned char *eomorig,const unsigned char *comp_dn,char *exp_dn,int length);
 CDECL int dn_skipname(const unsigned char *msg,const unsigned char *eomorig);
 #endif
@@ -507,6 +502,8 @@ void Resolver::LookupSRV_RR()
    char *srv_name=string_alloca(strlen(service)+1+strlen(tproto)+1+strlen(hostname)+1);
    sprintf(srv_name,"_%s._%s.%s",service,tproto,hostname);
 
+   int retries=0;
+   int max_retries=ResMgr::Query("dns:max-retries",hostname);
    int len;
    for(;;)
    {
@@ -523,6 +520,8 @@ void Resolver::LookupSRV_RR()
 #ifdef HAVE_H_ERRNO
       if(h_errno!=TRY_AGAIN)
 	 return;
+      if(++retries>=max_retries && max_retries)
+	 return;
       time_t t=time(0);
       if(t-try_time<5)
 	 sleep(5-(t-try_time));
@@ -699,6 +698,8 @@ void Resolver::LookupOne(const char *name)
 
    ParseOrder(order,af_order);
 
+   int retries=0;
+   int max_retries=ResMgr::Query("dns:max-retries",name);
    for(;;)
    {
       if(!use_fork)
@@ -710,9 +711,12 @@ void Resolver::LookupOne(const char *name)
 
       time(&try_time);
 
-#if defined(HAVE_GETADDRINFO) && INET6 \
-   && !defined(HAVE_GETHOSTBYNAME2) \
-   && !defined(HAVE_GETIPNODEBYNAME)
+// Prefer getaddrinfo over gethostbyname2 and getipnodebyname, as
+// approach with multiple lookups works badly when host name is in hosts file
+// and no dns servers are reachable.
+#if defined(HAVE_GETADDRINFO) && INET6
+/* && !defined(HAVE_GETHOSTBYNAME2) \
+   && !defined(HAVE_GETIPNODEBYNAME) */
 
       // getaddrinfo support by Brandon Hume
       struct addrinfo	    *ainfo=0,
@@ -723,15 +727,11 @@ void Resolver::LookupOne(const char *name)
       struct sockaddr_in    *inet_addr;
       struct sockaddr_in6   *inet6_addr;
       const char	    *addr_data;
+      int		    addr_len;
 
+      memset(&a_hint, 0, sizeof(a_hint));
       a_hint.ai_flags	    = AI_PASSIVE;
       a_hint.ai_family	    = PF_UNSPEC;
-      a_hint.ai_socktype    = 0;
-      a_hint.ai_protocol    = 0;
-      a_hint.ai_addrlen	    = 0;
-      a_hint.ai_canonname   = NULL;
-      a_hint.ai_addr	    = NULL;
-      a_hint.ai_next	    = NULL;
 
       ainfo_res	= getaddrinfo(name, NULL, &a_hint, &ainfo);
 
@@ -750,18 +750,19 @@ void Resolver::LookupOne(const char *name)
 	       switch(a_res->ai_family)
 	       {
 	       case AF_INET:
-		  inet_addr	= (sockaddr_in *)sockname;
-		  addr_data	= (const char *)&inet_addr->sin_addr.s_addr;
+		  inet_addr   = (sockaddr_in *)sockname;
+		  addr_data   = (const char *)&(inet_addr->sin_addr.s_addr);
+		  addr_len    = sizeof(inet_addr->sin_addr.s_addr);
 		  break;
 	       case AF_INET6:
-		  inet6_addr	= (sockaddr_in6 *)sockname;
-		  addr_data	= (const char *)inet6_addr->sin6_addr.s6_addr;
+		  inet6_addr  = (sockaddr_in6 *)sockname;
+		  addr_data   = (const char *)&(inet6_addr->sin6_addr.s6_addr);
+		  addr_len    = sizeof(inet6_addr->sin6_addr.s6_addr);
 		  break;
 	       default:
 		  continue;
 	       }
-
-	       AddAddress(a_res->ai_family, addr_data, a_res->ai_addrlen);
+	       AddAddress(a_res->ai_family, addr_data, addr_len);
 	    }
 	 }
 
@@ -769,7 +770,8 @@ void Resolver::LookupOne(const char *name)
 	 break;
       }
 
-      if(ainfo_res != EAI_AGAIN)
+      if(ainfo_res != EAI_AGAIN
+      || (++retries>=max_retries && max_retries))
       {
 	 error = gai_strerror(ainfo_res);
 	 break;
@@ -796,6 +798,7 @@ void Resolver::LookupOne(const char *name)
 	 ha=gethostbyname(name);
       else
       {
+	 retries=0;
 	 af_index++;
 	 continue;
       }
@@ -806,6 +809,7 @@ void Resolver::LookupOne(const char *name)
 	 const char * const *a;
 	 for(a=ha->h_addr_list; *a; a++)
 	    AddAddress(ha->h_addrtype, *a, ha->h_length);
+	 retries=0;
 	 af_index++;
 # if defined(HAVE_GETIPNODEBYNAME)
 	 freehostent(ha);
@@ -814,7 +818,8 @@ void Resolver::LookupOne(const char *name)
       }
 
 # ifdef HAVE_H_ERRNO
-      if(h_errno!=TRY_AGAIN)
+      if(h_errno!=TRY_AGAIN
+      || (++retries>=max_retries && max_retries))
 # endif
       {
 	 if(error==0)
@@ -825,6 +830,7 @@ void Resolver::LookupOne(const char *name)
 	    error=_("Host name lookup failure");
 # endif
 	 }
+	 retries=0;
 	 af_index++;
 	 continue; // try other address families
       }
@@ -896,58 +902,46 @@ flush:
 
 void Resolver::Reconfig(const char *name)
 {
-   timeout = ResMgr::Query("dns:fatal-timeout",hostname);
    if(!name || strncmp(name,"dns:",4))
       return;
-   if(cache)
-      cache->Clear();
 }
 
 
-
 ResolverCache::ResolverCache()
+   : Cache(ResMgr::FindRes("dns:cache-size"),ResMgr::FindRes("dns:cache-enable"))
 {
-   chain=0;
+}
+ResolverCacheEntry *ResolverCache::Find(const char *h,const char *p,const char *defp,const char *ser,const char *pr)
+{
+   for(ResolverCacheEntry *c=IterateFirst(); c; c=IterateNext())
+   {
+      if(c->Matches(h,p,defp,ser,pr))
+	 return c;
+   }
+   return 0;
 }
 void ResolverCache::Add(const char *h,const char *p,const char *defp,
 	 const char *ser,const char *pr,const sockaddr_u *a,int n)
 {
-   Entry **ptr=FindPtr(h,p,defp,ser,pr);
-   if(ptr && *ptr)
+   Trim();
+   ResolverCacheEntry *c=Find(h,p,defp,ser,pr);
+   if(c)
+      c->SetData(a,n);
+   else
    {
-      // delete old
-      Entry *next=(*ptr)->next;
-      delete *ptr;
-      *ptr=next;
+      if(!IsEnabled(h))
+	 return;
+      chain=new ResolverCacheEntry(chain,h,p,defp,ser,pr,a,n);
    }
-   chain=new Entry(chain,h,p,defp,ser,pr,a,n);
 }
-ResolverCache::Entry **ResolverCache::FindPtr(const char *h,const char *p,
+bool ResolverCacheEntryLoc::Matches(const char *h,const char *p,
 	 const char *defp,const char *ser,const char *pr)
 {
-   CacheCheck();
-   Entry **scan=&chain;
-   while(*scan)
-   {
-      Entry *s=*scan;
-      if(!xstrcasecmp(s->hostname,h)
-      && !xstrcmp(s->portname,p)
-      && !xstrcmp(s->defport,defp)
-      && !xstrcmp(s->service,ser)
-      && !xstrcmp(s->proto,pr))
-	 return scan;
-      scan=&s->next;
-   }
-   return 0;
-}
-void ResolverCache::Clear()
-{
-   while(chain)
-   {
-      Entry *next=chain->next;
-      delete chain;
-      chain=next;
-   }
+   return (!xstrcasecmp(hostname,h)
+      && !xstrcmp(portname,p)
+      && !xstrcmp(defport,defp)
+      && !xstrcmp(service,ser)
+      && !xstrcmp(proto,pr));
 }
 void ResolverCache::Find(const char *h,const char *p,const char *defp,
 	 const char *ser,const char *pr,const sockaddr_u **a,int *n)
@@ -956,36 +950,17 @@ void ResolverCache::Find(const char *h,const char *p,const char *defp,
    *n=0;
 
    // if cache is disabled for this host, return nothing.
-   if(!ResMgr::QueryBool("dns:cache-enable",h))
+   if(!IsEnabled(h))
       return;
 
-   Entry **ptr=FindPtr(h,p,defp,ser,pr);
-   if(ptr && *ptr)
+   ResolverCacheEntry *c=Find(h,p,defp,ser,pr);
+   if(c)
    {
-      Entry *s=*ptr;
-      *a=s->addr;
-      *n=s->addr_num;
-   }
-}
-
-// FIXME: this function can be speed-optimized.
-void ResolverCache::CacheCheck()
-{
-   int countlimit=ResMgr::Query("dns:cache-size",0);
-   int count=0;
-   Entry **scan=&chain;
-   while(*scan)
-   {
-      Entry *s=*scan;
-      TimeInterval expire((const char *)ResMgr::Query("dns:cache-expire",s->hostname));
-      if((!expire.IsInfty() && SMTask::now>=s->timestamp+expire.Seconds())
-      || (count>=countlimit))
+      if(c->Stopped())
       {
-	 *scan=s->next;
-	 delete s;
-	 continue;
+	 Trim();
+	 return;
       }
-      scan=&s->next;
-      count++;
+      c->GetData(a,n);
    }
 }
