@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.39 2005/10/18 12:35:23 lav Exp $ */
+/* $Id: SMTask.cc,v 1.47 2006/06/13 14:35:41 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -30,6 +30,7 @@
 #endif
 
 #include "SMTask.h"
+#include "Timer.h"
 #include "misc.h"
 
 SMTask	 *SMTask::chain=0;
@@ -60,6 +61,7 @@ SMTask::SMTask()
    suspended=false;
    suspended_slave=false;
    running=0;
+   ref_count=0;
    deleting=false;
    task_count++;
 #ifdef TASK_DEBUG
@@ -107,6 +109,7 @@ SMTask::~SMTask()
 #endif
    task_count--;
    assert(!running);
+   assert(!ref_count);
    // remove from the chain
    SMTask **scan=&chain;
    while(*scan)
@@ -126,7 +129,7 @@ void SMTask::Delete(SMTask *task)
    if(!task)
       return;
    task->deleting=true;
-   if(!task->running)
+   if(!task->running && !task->ref_count)
       delete task;
 }
 
@@ -157,12 +160,11 @@ int SMTask::Roll(SMTask *task)
    return m;
 }
 
-void SMTask::RollAll(int max_time)
+void SMTask::RollAll(const TimeInterval &max_time)
 {
-   time_t time_limit=now.UnixTime()+max_time;
+   Timer limit_timer(max_time);
    do { Schedule(); }
-   while(sched_total.GetTimeout()==0
-	 && (max_time==0 || now<time_limit));
+   while(sched_total.GetTimeout()==0 && !limit_timer.Stopped());
 }
 
 int SMTask::CollectGarbage()
@@ -174,7 +176,7 @@ int SMTask::CollectGarbage()
       SMTask *scan=chain;
       while(scan)
       {
-	 if(scan->running || !scan->deleting)
+	 if(scan->running || !scan->deleting || scan->ref_count)
 	 {
 	    scan=scan->next;
 	    continue;
@@ -211,6 +213,10 @@ void SMTask::Schedule()
    // get time once and assume Do() don't take much time
    UpdateNow();
 
+   int timer_timeout=Timer::GetTimeout();
+   if(timer_timeout>=0)
+      sched_total.SetTimeout(timer_timeout);
+
    bool repeat=false;
    scan=chain;
    while(scan)
@@ -254,6 +260,7 @@ void SMTask::Schedule()
 void SMTask::ReconfigAll(const char *name)
 {
    UpdateNow();
+   Timer::ReconfigAll(name);
    for(SMTask *scan=chain; scan; scan=scan->next)
       scan->Reconfig(name);
    sched_total.SetTimeout(0);  // for new values handling
@@ -282,8 +289,8 @@ int SMTask::TaskCount()
 
 void SMTask::Cleanup()
 {
-  CollectGarbage();
-  delete init_task;
+   CollectGarbage();
+   delete init_task;
 }
 
 #include <errno.h>
@@ -322,3 +329,14 @@ bool SMTask::TemporaryNetworkError(int err)
 {
    return temporary_network_error(err);
 }
+
+void SMTask::PrintTasks()
+{
+   for(SMTask *scan=chain; scan; scan=scan->next)
+   {
+      const char *c=scan->GetLogContext();
+      if(!c) c="";
+      printf("%p\t%c%c%c\t%s\n",scan,scan->running?'R':' ',
+	 scan->suspended?'S':' ',scan->deleting?'D':' ',c);
+   }
+}
