@@ -18,26 +18,38 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* $Id: SignalHook.h,v 1.6 2004/09/06 09:19:57 lav Exp $ */
+
 #ifndef SIGNALHOOK_H
 #define SIGNALHOOK_H
 
 #include <signal.h>
 
+typedef void (*signal_handler)(int);
+
 class SignalHook
 {
-   static int counts[256];
-   static void (*(old_handlers[256]))(int);
+   static int *counts;
+   static struct sigaction *old_handlers;
+   static bool *old_saved;
+
    static void cnt_handler(int sig);
-   static void set_signal(int sig,void (*handler)(int));
+   static void set_signal(int sig,signal_handler handler);
 public:
-   static void DoCount(int sig) { set_signal(sig,cnt_handler); }
+   static void DoCount(int sig) { set_signal(sig,&SignalHook::cnt_handler); }
    static int GetCount(int sig) { return counts[sig]; }
    static void ResetCount(int sig) { counts[sig]=0; }
    static void IncreaseCount(int sig) { counts[sig]++; }
-   static void AddHook(int sig,void (*)(int));
-   static void Ignore(int sig) { set_signal(sig,(void (*)(int))SIG_IGN); }
+   static void Handle(int sig,void (*h)(int)) { set_signal(sig,h); }
+   static void Ignore(int sig)  { set_signal(sig,(signal_handler)SIG_IGN); }
+   static void Default(int sig) { set_signal(sig,(signal_handler)SIG_DFL); }
    static void Restore(int sig);
+   static void Block(int sig);
+   static void Unblock(int sig);
    static void RestoreAll();
+
+   static void ClassInit();
+   static void Cleanup();
 };
 
 #endif//SIGNALHOOK_H
