@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.32 2004/06/12 07:55:43 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.36 2006/07/24 08:50:20 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -78,7 +78,9 @@ StatusLine::~StatusLine()
 
 void StatusLine::Clear(bool title_also)
 {
-   update("");
+   const char *empty="";
+   update_timer.Stop();
+   ShowN(&empty,1);
    update_delayed=false;
    update_timer.SetMilliSeconds(20);
 
@@ -184,13 +186,15 @@ void StatusLine::update(const char *const *newstr,int newstr_height)
    }
 
    int w=GetWidth();
-   int mbflags=MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE;
+   int mbflags=0;
 
    if(newstr_height>LastHeight)
       newstr_height=LastHeight;
 
    // clear old extra lines. Assume we are at beginning of last shown line.
    int j=shown.Count();
+   if(!prev_line)    // if there is no way to go up, show a single line only.
+      j=newstr_height=1;
    int i=j-newstr_height;
    char *spaces=string_alloca(w+1);
    memset(spaces,' ',w);
