@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: bookmark.h,v 1.3 2000/11/14 12:16:58 lav Exp $ */
+/* $Id: bookmark.h,v 1.6 2005/06/23 06:39:03 lav Exp $ */
 
 #ifndef BOOKMARK_H
 #define BOOKMARK_H
@@ -39,11 +39,15 @@ class Bookmark : public KeyValueDB
    void PreModify();
    void PostModify();
    void Close();
+   void AutoSync();
 public:
    void Add(const char *id,const char *value);
    void Remove(const char *id);
    const char *Lookup(const char *id);
    char *Format();
+   char *FormatHidePasswords();
+   void UserLoad() { Load(); Close(); }
+   void UserSave();
 
    Bookmark();
    ~Bookmark();
@@ -55,4 +59,6 @@ public:
       }
 };
 
+extern Bookmark lftp_bookmarks;
+
 #endif //BOOKMARK_H
