@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.74 2006/06/14 05:35:33 lav Exp $ */
+/* $Id: lftp.cc,v 1.76 2006/07/26 13:18:40 lav Exp $ */
 
 #include <config.h>
 
@@ -45,7 +45,6 @@
 #include "DummyProto.h"
 #include "ResMgr.h"
 #include "LsCache.h"
-#include "DirColors.h"
 #include "IdNameCache.h"
 #include "LocalDir.h"
 #include "ConnectionSlot.h"
@@ -426,10 +425,14 @@ static void tty_clear()
       top_exec->pre_stdout();
 }
 
+char *program_name;
+
 int   main(int argc,char **argv)
 {
+   program_name=argv[0];
+
 #ifdef SOCKS4
-   SOCKSinit(argv[0]);
+   SOCKSinit(program_name);
 #endif
 
    setlocale (LC_ALL, "");
@@ -503,7 +506,6 @@ int   main(int argc,char **argv)
    SessionPool::ClearAll();
    FileAccess::ClassCleanup();
    ProcWait::DeleteAll();
-   DirColors::DeleteInstance();
    IdNameCacheCleanup();
    SignalHook::Cleanup();
    Log::Cleanup();
