@@ -18,15 +18,20 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp_rl.c,v 1.8 2001/09/10 06:17:10 lav Exp $ */
+/* $Id: lftp_rl.c,v 1.15 2005/11/10 06:43:44 lav Exp $ */
 
 #include <config.h>
 #include <stdio.h>
 #include <readline/readline.h>
 #include <readline/history.h>
+#include <stdlib.h>
+#include <string.h>
 #include "lftp_rl.h"
 #include "xalloca.h"
 
+/* complete.cc */
+void lftp_line_complete();
+
 void lftp_add_history_nodups(const char *cmd_buf)
 {
    HIST_ENTRY *temp;
@@ -39,7 +44,12 @@ void lftp_add_history_nodups(const char *cmd_buf)
 
 char *lftp_readline(const char *prompt)
 {
-   return readline(prompt);
+   char *ret = readline(prompt);
+   /* Tell completion that we don't need completion data anymore;
+    * it might be taking a good chunk of memory. */
+   lftp_line_complete();
+
+   return ret;
 }
 
 int lftp_history_expand(const char *what, char **where)
@@ -124,6 +134,15 @@ char **lftp_rl_completion_matches(const char *text,char *(*compentry)(const char
    return rl_completion_matches(text,compentry);
 }
 
+void completion_display_list (char **matches, int len);
+
+void lftp_rl_display_match_list (char **matches, int len, int max)
+{
+   printf("\n"); /* get off the input line */
+   completion_display_list(matches, len);
+   rl_forced_update_display(); /* redraw input line */
+}
+
 void lftp_rl_init(
    const char *readline_name,
    char **(*attempted_completion_function)(const char *,int,int),
@@ -133,7 +152,7 @@ void lftp_rl_init(
    const char *filename_quote_characters,
    char *(*filename_quoting_function)(char *,int,char *),
    char *(*filename_dequoting_function)(char *,int),
-   int (*char_is_quoted_p)(char *,int))
+   int (*char_is_quoted_p)(const char *,int))
 {
    rl_readline_name                  =readline_name;
    rl_attempted_completion_function  =attempted_completion_function;
@@ -144,6 +163,8 @@ void lftp_rl_init(
    rl_filename_quoting_function      =filename_quoting_function;
    rl_filename_dequoting_function    =filename_dequoting_function;
    rl_char_is_quoted_p               =char_is_quoted_p;
+
+   rl_completion_display_matches_hook = lftp_rl_display_match_list;
 }
 
 void lftp_rl_add_defun(const char *name,int (*func)(int,int),int key)
@@ -156,3 +177,31 @@ void lftp_rl_bind(const char *key,const char *func)
    sprintf(line,"%s: %s",key,func);
    rl_parse_and_bind(line);
 }
+
+void lftp_rl_set_prompt(const char *p)
+{
+   rl_set_prompt(p);
+}
+
+extern char *get_lftp_home();
+
+static char *lftp_history_file;
+void lftp_rl_read_history()
+{
+   if(!lftp_history_file)
+   {
+      const char *add="/rl_history";
+      const char *home=get_lftp_home();
+      if(!home)
+	 return;
+      lftp_history_file=(char*)malloc(strlen(home)+strlen(add)+1);
+      strcat(strcpy(lftp_history_file,home),add);
+   }
+   read_history(lftp_history_file);
+}
+void lftp_rl_write_history()
+{
+   if(!lftp_history_file)
+      return;
+   write_history(lftp_history_file);
+}
