@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.80 2005/12/27 12:16:59 lav Exp $ */
+/* $Id: misc.cc,v 1.82 2006/07/24 08:50:22 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -457,7 +457,7 @@ const char *squeeze_file_name(const char *name,int w)
 {
    static char *buf;
    static int buf_len;
-   int mbflags=MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE;
+   int mbflags=0;
 
    name=url::remove_password(name);
 
