@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: pgetJob.cc,v 1.38 2006/07/03 13:23:56 lav Exp $ */
+/* $Id: pgetJob.cc,v 1.39 2006/07/05 17:17:01 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -319,9 +319,13 @@ void pgetJob::NextFile()
    total_eta=-1;
 
    xfree(status_file);
-   status_file=xasprintf("%s.lftp-pget-status",local->full_name);
-   if(pget_cont)
-      LoadStatus0();
+   status_file=0;
+   if(local && local->full_name)
+   {
+      status_file=xasprintf("%s.lftp-pget-status",local->full_name);
+      if(pget_cont)
+	 LoadStatus0();
+   }
 }
 
 pgetJob::ChunkXfer *pgetJob::NewChunk(const char *remote,FDStream *local,off_t start,off_t limit)
