@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.133 2006/06/30 14:10:30 lav Exp $ */
+/* $Id: resource.cc,v 1.135 2006/08/08 05:41:55 lav Exp $ */
 
 #include <config.h>
 
@@ -220,6 +220,7 @@ static ResType lftp_vars[] = {
    {"ftp:ssl-auth",		 "TLS",   AuthArgValidate,0},
    {"ftp:ssl-allow-anonymous",	 "no",	  ResMgr::BoolValidate,0},
    {"ftp:ssl-use-ccc",		 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:ssl-data-use-keys",	 "yes",	  ResMgr::BoolValidate,0},
    {"ftps:initial-prot",	 "",	  ProtValidate,0},
 #endif
    {"ftp:stat-interval",	 "1",	  ResMgr::TimeIntervalValidate,0},
@@ -300,6 +301,7 @@ static ResType lftp_vars[] = {
    {"mirror:parallel-directories", "yes", ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:parallel-transfer-count", "1",ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"mirror:exclude-regex",	 "(^|/)(\\.in\\.|\\.nfs)",ResMgr::ERegExpValidate,ResMgr::NoClosure},
+   {"mirror:include-regex",	 "",	  ResMgr::ERegExpValidate,ResMgr::NoClosure},
    {"mirror:use-pget-n",	 "1",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
    {"mirror:set-permissions",	 "yes",   ResMgr::BoolValidate,ResMgr::NoClosure},
    {"mirror:dereference",	 "no",    ResMgr::BoolValidate,ResMgr::NoClosure},
